# This file was auto-generated by Fern from our API Definition.

import typing

from .create_flows_vault_connection_sendgrid_api_key import CreateFlowsVaultConnectionSendgridApiKey
from .create_flows_vault_connection_sendgrid_uninitialized import CreateFlowsVaultConnectionSendgridUninitialized

CreateFlowsVaultConnectionSendgrid = typing.Union[
    CreateFlowsVaultConnectionSendgridApiKey, CreateFlowsVaultConnectionSendgridUninitialized
]



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionRealms = typing.List[str]
"""
Defines the realms for which the connection will be used (ie: email domains). If the array is empty or the property is not specified, the connection name will be added as realm.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .client_session_transfer_allowed_authentication_methods_enum import (
    ClientSessionTransferAllowedAuthenticationMethodsEnum,
)
from .client_session_transfer_delegation_configuration import ClientSessionTransferDelegationConfiguration
from .client_session_transfer_device_binding_enum import ClientSessionTransferDeviceBindingEnum


class ClientSessionTransferConfiguration(UniversalBaseModel):
    """
    Native to Web SSO Configuration
    """

    can_create_session_transfer_token: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether an app can issue a Session Transfer Token through Token Exchange. If set to 'false', the app will not be able to issue a Session Transfer Token. Usually configured in the native application. Default value is `false`.
    """

    enforce_cascade_revocation: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether revoking the parent Refresh Token that initiated a Native to Web flow and was used to issue a Session Transfer Token should trigger a cascade revocation affecting its dependent child entities. Usually configured in the native application. Default value is `true`, applicable only in Native to Web SSO context.
    """

    allowed_authentication_methods: typing.Optional[
        typing.List[ClientSessionTransferAllowedAuthenticationMethodsEnum]
    ] = pydantic.Field(default=None)
    """
    Indicates whether an app can create a session from a Session Transfer Token received via indicated methods. Can include `cookie` and/or `query`. Usually configured in the web application. Default value is an empty array [].
    """

    enforce_device_binding: typing.Optional[ClientSessionTransferDeviceBindingEnum] = None
    allow_refresh_token: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether Refresh Tokens are allowed to be issued when authenticating with a Session Transfer Token. Usually configured in the web application. Default value is `false`.
    """

    enforce_online_refresh_tokens: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether Refresh Tokens created during a Native to Web session are tied to that session's lifetime. This determines if such refresh tokens should be automatically revoked when their corresponding sessions are. Usually configured in the web application. Default value is `true`, applicable only in Native to Web SSO context.
    """

    delegation: typing.Optional[ClientSessionTransferDelegationConfiguration] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.custom_signing_key_jwk import CustomSigningKeyJwk
from ...types.get_custom_signing_keys_response_content import GetCustomSigningKeysResponseContent
from ...types.set_custom_signing_keys_response_content import SetCustomSigningKeysResponseContent
from .raw_client import AsyncRawCustomSigningClient, RawCustomSigningClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomSigningClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCustomSigningClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCustomSigningClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCustomSigningClient
        """
        return self._raw_client

    def get(self, *, request_options: typing.Optional[RequestOptions] = None) -> GetCustomSigningKeysResponseContent:
        """
        Get entire jwks representation of custom signing keys.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomSigningKeysResponseContent
            Custom signing keys were successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.keys.custom_signing.get()
        """
        _response = self._raw_client.get(request_options=request_options)
        return _response.data

    def set(
        self, *, keys: typing.Sequence[CustomSigningKeyJwk], request_options: typing.Optional[RequestOptions] = None
    ) -> SetCustomSigningKeysResponseContent:
        """
        Create or replace entire jwks representation of custom signing keys.

        Parameters
        ----------
        keys : typing.Sequence[CustomSigningKeyJwk]
            An array of custom public signing keys.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SetCustomSigningKeysResponseContent
            Custom signing keys were successfully created or replaced.

        Examples
        --------
        from auth0 import Auth0, CustomSigningKeyJwk

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.keys.custom_signing.set(
            keys=[
                CustomSigningKeyJwk(
                    kty="EC",
                )
            ],
        )
        """
        _response = self._raw_client.set(keys=keys, request_options=request_options)
        return _response.data

    def delete(self, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete entire jwks representation of custom signing keys.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.keys.custom_signing.delete()
        """
        _response = self._raw_client.delete(request_options=request_options)
        return _response.data


class AsyncCustomSigningClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCustomSigningClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCustomSigningClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCustomSigningClient
        """
        return self._raw_client

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetCustomSigningKeysResponseContent:
        """
        Get entire jwks representation of custom signing keys.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomSigningKeysResponseContent
            Custom signing keys were successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.keys.custom_signing.get()


        asyncio.run(main())
        """
        _response = await self._raw_client.get(request_options=request_options)
        return _response.data

    async def set(
        self, *, keys: typing.Sequence[CustomSigningKeyJwk], request_options: typing.Optional[RequestOptions] = None
    ) -> SetCustomSigningKeysResponseContent:
        """
        Create or replace entire jwks representation of custom signing keys.

        Parameters
        ----------
        keys : typing.Sequence[CustomSigningKeyJwk]
            An array of custom public signing keys.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SetCustomSigningKeysResponseContent
            Custom signing keys were successfully created or replaced.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, CustomSigningKeyJwk

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.keys.custom_signing.set(
                keys=[
                    CustomSigningKeyJwk(
                        kty="EC",
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.set(keys=keys, request_options=request_options)
        return _response.data

    async def delete(self, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete entire jwks representation of custom signing keys.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.keys.custom_signing.delete()


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .action_execution_result import ActionExecutionResult
from .action_execution_status_enum import ActionExecutionStatusEnum
from .action_trigger_type_enum import ActionTriggerTypeEnum


class GetActionExecutionResponseContent(UniversalBaseModel):
    """
    The result of a specific execution of a trigger.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID identifies this specific execution simulation. These IDs would resemble real executions in production.
    """

    trigger_id: typing.Optional[ActionTriggerTypeEnum] = None
    status: typing.Optional[ActionExecutionStatusEnum] = None
    results: typing.Optional[typing.List[ActionExecutionResult]] = None
    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time that the execution was started.
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time that the exeution finished executing.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_flow_map_value_params_fallback_object import FlowActionFlowMapValueParamsFallbackObject

FlowActionFlowMapValueParamsFallback = typing.Union[
    str, float, FlowActionFlowMapValueParamsFallbackObject, typing.List[typing.Any]
]



# This file was auto-generated by Fern from our API Definition.

from .connection_https_url_with_http_fallback import ConnectionHttpsUrlWithHttpFallback

ConnectionCommunityBaseUrlSalesforce = ConnectionHttpsUrlWithHttpFallback
"""
The base URL of your Salesforce Community (Experience Cloud) site. When specified, authentication flows will use this Community URL instead of the standard Salesforce login page, enabling users to authenticate through your branded Community portal.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

PromptGroupNameEnum = typing.Union[
    typing.Literal[
        "login",
        "login-id",
        "login-password",
        "login-passwordless",
        "login-email-verification",
        "signup",
        "signup-id",
        "signup-password",
        "phone-identifier-enrollment",
        "phone-identifier-challenge",
        "email-identifier-challenge",
        "reset-password",
        "custom-form",
        "consent",
        "customized-consent",
        "logout",
        "mfa-push",
        "mfa-otp",
        "mfa-voice",
        "mfa-phone",
        "mfa-webauthn",
        "mfa-sms",
        "mfa-email",
        "mfa-recovery-code",
        "mfa",
        "status",
        "device-flow",
        "email-verification",
        "email-otp-challenge",
        "organizations",
        "invitation",
        "common",
        "passkeys",
        "captcha",
        "brute-force-protection",
        "async-approval-flow",
        "confirmation",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .client_jwt_configuration_scopes import ClientJwtConfigurationScopes
from .signing_algorithm_enum import SigningAlgorithmEnum


class ClientJwtConfiguration(UniversalBaseModel):
    """
    Configuration related to JWTs for the client.
    """

    lifetime_in_seconds: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of seconds the JWT will be valid for (affects `exp` claim).
    """

    secret_encoded: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the client secret is base64 encoded (true) or unencoded (false).
    """

    scopes: typing.Optional[ClientJwtConfigurationScopes] = None
    alg: typing.Optional[SigningAlgorithmEnum] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .create_export_users_fields import CreateExportUsersFields
from .job_file_format_enum import JobFileFormatEnum


class CreateExportUsersResponseContent(UniversalBaseModel):
    status: str = pydantic.Field()
    """
    Status of this job.
    """

    type: str = pydantic.Field()
    """
    Type of job this is.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    When this job was created.
    """

    id: str = pydantic.Field()
    """
    ID of this job.
    """

    connection_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    connection_id of the connection from which users will be exported.
    """

    format: typing.Optional[JobFileFormatEnum] = None
    limit: typing.Optional[int] = pydantic.Field(default=None)
    """
    Limit the number of records.
    """

    fields: typing.Optional[typing.List[CreateExportUsersFields]] = pydantic.Field(default=None)
    """
    List of fields to be included in the CSV. Defaults to a predefined set of fields.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .connection_https_url_with_http_fallback_2048 import ConnectionHttpsUrlWithHttpFallback2048

ConnectionSignOutEndpointSaml = ConnectionHttpsUrlWithHttpFallback2048
"""
Identity provider's SAML SingleLogoutService endpoint URL where Auth0 sends logout requests for federated sign-out. When not provided, defaults to signInEndpoint. Only used if disableSignout is false.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .custom_signing_key_algorithm_enum import CustomSigningKeyAlgorithmEnum
from .custom_signing_key_curve_enum import CustomSigningKeyCurveEnum
from .custom_signing_key_operation_enum import CustomSigningKeyOperationEnum
from .custom_signing_key_type_enum import CustomSigningKeyTypeEnum
from .custom_signing_key_use_enum import CustomSigningKeyUseEnum


class CustomSigningKeyJwk(UniversalBaseModel):
    """
    JWK representing a custom public signing key.
    """

    kty: CustomSigningKeyTypeEnum
    kid: typing.Optional[str] = pydantic.Field(default=None)
    """
    Key identifier
    """

    use: typing.Optional[CustomSigningKeyUseEnum] = None
    key_ops: typing.Optional[typing.List[CustomSigningKeyOperationEnum]] = pydantic.Field(default=None)
    """
    Key operations
    """

    alg: typing.Optional[CustomSigningKeyAlgorithmEnum] = None
    n: typing.Optional[str] = pydantic.Field(default=None)
    """
    Key modulus
    """

    e: typing.Optional[str] = pydantic.Field(default=None)
    """
    Key exponent
    """

    crv: typing.Optional[CustomSigningKeyCurveEnum] = None
    x: typing.Optional[str] = pydantic.Field(default=None)
    """
    X coordinate
    """

    y: typing.Optional[str] = pydantic.Field(default=None)
    """
    Y coordinate
    """

    x_5_u: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="x5u"), pydantic.Field(alias="x5u", description="X.509 URL")
    ] = None
    """
    X.509 URL
    """

    x_5_c: typing_extensions.Annotated[
        typing.Optional[typing.List[str]],
        FieldMetadata(alias="x5c"),
        pydantic.Field(alias="x5c", description="X.509 certificate chain"),
    ] = None
    """
    X.509 certificate chain
    """

    x_5_t: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="x5t"),
        pydantic.Field(alias="x5t", description="X.509 certificate SHA-1 thumbprint"),
    ] = None
    """
    X.509 certificate SHA-1 thumbprint
    """

    x_5_t_s_256: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="x5t#S256"),
        pydantic.Field(alias="x5t#S256", description="X.509 certificate SHA-256 thumbprint"),
    ] = None
    """
    X.509 certificate SHA-256 thumbprint
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .public_key_credential import PublicKeyCredential

ClientCreateAuthenticationMethodPrivateKeyJwtCredentials = typing.List[PublicKeyCredential]
"""
Fully defined credentials that will be enabled on the client for Private Key JWT authentication.
"""



# This file was auto-generated by Fern from our API Definition.

from .connection_metadata_xml import ConnectionMetadataXml

ConnectionMetadataXmlSaml = ConnectionMetadataXml
"""
SAML metadata XML document from the identity provider. When provided, automatically parsed to extract signInEndpoint, signOutEndpoint, signingCert, signSAMLRequest, and protocolBinding. Deleted after parsing to avoid persisting large documents. Not persisted to database - deleted after parsing metadata. Use metadataUrl OR metadataXml, not both.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .breached_password_detection_pre_user_registration_shields_enum import (
    BreachedPasswordDetectionPreUserRegistrationShieldsEnum,
)


class BreachedPasswordDetectionPreUserRegistrationStage(UniversalBaseModel):
    shields: typing.Optional[typing.List[BreachedPasswordDetectionPreUserRegistrationShieldsEnum]] = pydantic.Field(
        default=None
    )
    """
    Action to take when a breached password is detected during a signup.
                  Possible values: <code>block</code>, <code>admin_notification</code>.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionDomainAliasesAzureAd = typing.List[str]
"""
Alternative domain names associated with this Azure AD tenant. Allows users from multiple verified domains to authenticate through this connection. Can be an array of domain strings.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .email_template_name_enum import EmailTemplateNameEnum


class CreateEmailTemplateResponseContent(UniversalBaseModel):
    template: EmailTemplateNameEnum
    body: typing.Optional[str] = pydantic.Field(default=None)
    """
    Body of the email template.
    """

    from_: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="from"),
        pydantic.Field(alias="from", description="Senders `from` email address."),
    ] = None
    """
    Senders `from` email address.
    """

    result_url: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="resultUrl"),
        pydantic.Field(alias="resultUrl", description="URL to redirect the user to after a successful action."),
    ] = None
    """
    URL to redirect the user to after a successful action.
    """

    subject: typing.Optional[str] = pydantic.Field(default=None)
    """
    Subject line of the email.
    """

    syntax: typing.Optional[str] = pydantic.Field(default=None)
    """
    Syntax of the template body.
    """

    url_lifetime_in_seconds: typing_extensions.Annotated[
        typing.Optional[float],
        FieldMetadata(alias="urlLifetimeInSeconds"),
        pydantic.Field(
            alias="urlLifetimeInSeconds",
            description="Lifetime in seconds that the link within the email will be valid for.",
        ),
    ] = None
    """
    Lifetime in seconds that the link within the email will be valid for.
    """

    include_email_in_redirect: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="includeEmailInRedirect"),
        pydantic.Field(
            alias="includeEmailInRedirect",
            description="Whether the `reset_email` and `verify_email` templates should include the user's email address as the `email` parameter in the returnUrl (true) or whether no email address should be included in the redirect (false). Defaults to true.",
        ),
    ] = None
    """
    Whether the `reset_email` and `verify_email` templates should include the user's email address as the `email` parameter in the returnUrl (true) or whether no email address should be included in the redirect (false). Defaults to true.
    """

    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the template is enabled (true) or disabled (false).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.connection_enabled_client import ConnectionEnabledClient
from ...types.get_connection_enabled_clients_response_content import GetConnectionEnabledClientsResponseContent
from ...types.update_enabled_client_connections_request_content import UpdateEnabledClientConnectionsRequestContent
from .raw_client import AsyncRawClientsClient, RawClientsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ClientsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawClientsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawClientsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawClientsClient
        """
        return self._raw_client

    def get(
        self,
        id: str,
        *,
        take: typing.Optional[int] = 50,
        from_: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ConnectionEnabledClient, GetConnectionEnabledClientsResponseContent]:
        """
        Retrieve all clients that have the specified [connection](https://auth0.com/docs/authenticate/identity-providers) enabled.

        **Note**: The first time you call this endpoint, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no further results are remaining.

        Parameters
        ----------
        id : str
            The id of the connection for which enabled clients are to be retrieved

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ConnectionEnabledClient, GetConnectionEnabledClientsResponseContent]
            Success

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.connections.clients.get(
            id="id",
            take=1,
            from_="from",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.get(id, take=take, from_=from_, request_options=request_options)

    def update(
        self,
        id: str,
        *,
        request: UpdateEnabledClientConnectionsRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        id : str
            The id of the connection to modify

        request : UpdateEnabledClientConnectionsRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0, UpdateEnabledClientConnectionsRequestContentItem

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.clients.update(
            id="id",
            request=[
                UpdateEnabledClientConnectionsRequestContentItem(
                    client_id="client_id",
                    status=True,
                )
            ],
        )
        """
        _response = self._raw_client.update(id, request=request, request_options=request_options)
        return _response.data


class AsyncClientsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawClientsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawClientsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawClientsClient
        """
        return self._raw_client

    async def get(
        self,
        id: str,
        *,
        take: typing.Optional[int] = 50,
        from_: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ConnectionEnabledClient, GetConnectionEnabledClientsResponseContent]:
        """
        Retrieve all clients that have the specified [connection](https://auth0.com/docs/authenticate/identity-providers) enabled.

        **Note**: The first time you call this endpoint, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no further results are remaining.

        Parameters
        ----------
        id : str
            The id of the connection for which enabled clients are to be retrieved

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ConnectionEnabledClient, GetConnectionEnabledClientsResponseContent]
            Success

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.connections.clients.get(
                id="id",
                take=1,
                from_="from",
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.get(id, take=take, from_=from_, request_options=request_options)

    async def update(
        self,
        id: str,
        *,
        request: UpdateEnabledClientConnectionsRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        id : str
            The id of the connection to modify

        request : UpdateEnabledClientConnectionsRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, UpdateEnabledClientConnectionsRequestContentItem

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.clients.update(
                id="id",
                request=[
                    UpdateEnabledClientConnectionsRequestContentItem(
                        client_id="client_id",
                        status=True,
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(id, request=request, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ClientAddonFirebase(UniversalBaseModel):
    """
    Google Firebase addon configuration.
    """

    secret: typing.Optional[str] = pydantic.Field(default=None)
    """
    Google Firebase Secret. (SDK 2 only).
    """

    private_key_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional ID of the private key to obtain kid header in the issued token (SDK v3+ tokens only).
    """

    private_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    Private Key for signing the token (SDK v3+ tokens only).
    """

    client_email: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the Service Account you have created (shown as `client_email` in the generated JSON file, SDK v3+ tokens only).
    """

    lifetime_in_seconds: typing.Optional[int] = pydantic.Field(default=None)
    """
    Optional expiration in seconds for the generated token. Defaults to 3600 seconds (SDK v3+ tokens only).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_keys_encryption_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "keys.encryption.list_.0"
    client = get_client(test_id)
    client.keys.encryption.list(
        page=1,
        per_page=1,
        include_totals=True,
    )
    verify_request_count(
        test_id, "GET", "/keys/encryption", {"page": "1", "per_page": "1", "include_totals": "true"}, 1
    )


def test_keys_encryption_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "keys.encryption.create.0"
    client = get_client(test_id)
    client.keys.encryption.create(
        type="customer-provided-root-key",
    )
    verify_request_count(test_id, "POST", "/keys/encryption", None, 1)


def test_keys_encryption_rekey() -> None:
    """Test rekey endpoint with WireMock"""
    test_id = "keys.encryption.rekey.0"
    client = get_client(test_id)
    client.keys.encryption.rekey()
    verify_request_count(test_id, "POST", "/keys/encryption/rekey", None, 1)


def test_keys_encryption_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "keys.encryption.get.0"
    client = get_client(test_id)
    client.keys.encryption.get(
        kid="kid",
    )
    verify_request_count(test_id, "GET", "/keys/encryption/kid", None, 1)


def test_keys_encryption_import_() -> None:
    """Test import endpoint with WireMock"""
    test_id = "keys.encryption.import_.0"
    client = get_client(test_id)
    client.keys.encryption.import_(
        kid="kid",
        wrapped_key="wrapped_key",
    )
    verify_request_count(test_id, "POST", "/keys/encryption/kid", None, 1)


def test_keys_encryption_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "keys.encryption.delete.0"
    client = get_client(test_id)
    client.keys.encryption.delete(
        kid="kid",
    )
    verify_request_count(test_id, "DELETE", "/keys/encryption/kid", None, 1)


def test_keys_encryption_create_public_wrapping_key() -> None:
    """Test createPublicWrappingKey endpoint with WireMock"""
    test_id = "keys.encryption.create_public_wrapping_key.0"
    client = get_client(test_id)
    client.keys.encryption.create_public_wrapping_key(
        kid="kid",
    )
    verify_request_count(test_id, "POST", "/keys/encryption/kid/wrapping-key", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from .connection_debug_saml import ConnectionDebugSaml
from .connection_destination_url_saml import ConnectionDestinationUrlSaml
from .connection_fields_map_saml import ConnectionFieldsMapSaml
from .connection_global_token_revocation_jwt_iss_saml import ConnectionGlobalTokenRevocationJwtIssSaml
from .connection_global_token_revocation_jwt_sub_saml import ConnectionGlobalTokenRevocationJwtSubSaml
from .connection_metadata_url_saml import ConnectionMetadataUrlSaml
from .connection_metadata_xml_saml import ConnectionMetadataXmlSaml
from .connection_options_common import ConnectionOptionsCommon
from .connection_options_common_saml import ConnectionOptionsCommonSaml
from .connection_options_deflate_saml import ConnectionOptionsDeflateSaml
from .connection_recipient_url_saml import ConnectionRecipientUrlSaml
from .connection_request_template_saml import ConnectionRequestTemplateSaml
from .connection_sign_out_endpoint_saml import ConnectionSignOutEndpointSaml
from .connection_signing_cert_saml import ConnectionSigningCertSaml
from .connection_signing_key_saml import ConnectionSigningKeySaml
from .connection_user_id_attribute_saml import ConnectionUserIdAttributeSaml


class ConnectionOptionsSaml(ConnectionOptionsCommonSaml, ConnectionOptionsCommon):
    """
    Options for the 'samlp' connection
    """

    debug: typing.Optional[ConnectionDebugSaml] = None
    deflate: typing.Optional[ConnectionOptionsDeflateSaml] = None
    destination_url: typing_extensions.Annotated[
        typing.Optional[ConnectionDestinationUrlSaml],
        FieldMetadata(alias="destinationUrl"),
        pydantic.Field(alias="destinationUrl"),
    ] = None
    disable_signout: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="disableSignout"),
        pydantic.Field(
            alias="disableSignout",
            description="When true, disables sending SAML logout requests (SingleLogoutService) to the identity provider during user sign-out. The user will be logged out of Auth0 but will remain logged into the identity provider. Defaults to false (federated logout enabled).",
        ),
    ] = None
    """
    When true, disables sending SAML logout requests (SingleLogoutService) to the identity provider during user sign-out. The user will be logged out of Auth0 but will remain logged into the identity provider. Defaults to false (federated logout enabled).
    """

    fields_map: typing_extensions.Annotated[
        typing.Optional[ConnectionFieldsMapSaml], FieldMetadata(alias="fieldsMap"), pydantic.Field(alias="fieldsMap")
    ] = None
    global_token_revocation_jwt_iss: typing.Optional[ConnectionGlobalTokenRevocationJwtIssSaml] = None
    global_token_revocation_jwt_sub: typing.Optional[ConnectionGlobalTokenRevocationJwtSubSaml] = None
    metadata_url: typing_extensions.Annotated[
        typing.Optional[ConnectionMetadataUrlSaml],
        FieldMetadata(alias="metadataUrl"),
        pydantic.Field(alias="metadataUrl"),
    ] = None
    metadata_xml: typing_extensions.Annotated[
        typing.Optional[ConnectionMetadataXmlSaml],
        FieldMetadata(alias="metadataXml"),
        pydantic.Field(alias="metadataXml"),
    ] = None
    recipient_url: typing_extensions.Annotated[
        typing.Optional[ConnectionRecipientUrlSaml],
        FieldMetadata(alias="recipientUrl"),
        pydantic.Field(alias="recipientUrl"),
    ] = None
    request_template: typing_extensions.Annotated[
        typing.Optional[ConnectionRequestTemplateSaml],
        FieldMetadata(alias="requestTemplate"),
        pydantic.Field(alias="requestTemplate"),
    ] = None
    signing_cert: typing_extensions.Annotated[
        typing.Optional[ConnectionSigningCertSaml],
        FieldMetadata(alias="signingCert"),
        pydantic.Field(alias="signingCert"),
    ] = None
    signing_key: typing.Optional[ConnectionSigningKeySaml] = None
    sign_out_endpoint: typing_extensions.Annotated[
        typing.Optional[ConnectionSignOutEndpointSaml],
        FieldMetadata(alias="signOutEndpoint"),
        pydantic.Field(alias="signOutEndpoint"),
    ] = None
    user_id_attribute: typing.Optional[ConnectionUserIdAttributeSaml] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.create_guardian_enrollment_ticket_response_content import CreateGuardianEnrollmentTicketResponseContent
from ...types.get_guardian_enrollment_response_content import GetGuardianEnrollmentResponseContent
from ...types.guardian_enrollment_factor_enum import GuardianEnrollmentFactorEnum
from .raw_client import AsyncRawEnrollmentsClient, RawEnrollmentsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class EnrollmentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEnrollmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawEnrollmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEnrollmentsClient
        """
        return self._raw_client

    def create_ticket(
        self,
        *,
        user_id: str,
        email: typing.Optional[str] = OMIT,
        send_mail: typing.Optional[bool] = OMIT,
        email_locale: typing.Optional[str] = OMIT,
        factor: typing.Optional[GuardianEnrollmentFactorEnum] = OMIT,
        allow_multiple_enrollments: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateGuardianEnrollmentTicketResponseContent:
        """
        Create a [multi-factor authentication (MFA) enrollment ticket](https://auth0.com/docs/secure/multi-factor-authentication/auth0-guardian/create-custom-enrollment-tickets), and optionally send an email with the created ticket to a given user. Enrollment tickets can specify which factor users must enroll with or allow existing MFA users to enroll in additional factors.

        Parameters
        ----------
        user_id : str
            user_id for the enrollment ticket

        email : typing.Optional[str]
            alternate email to which the enrollment email will be sent. Optional - by default, the email will be sent to the user's default address

        send_mail : typing.Optional[bool]
            Send an email to the user to start the enrollment

        email_locale : typing.Optional[str]
            Optional. Specify the locale of the enrollment email. Used with send_email.

        factor : typing.Optional[GuardianEnrollmentFactorEnum]

        allow_multiple_enrollments : typing.Optional[bool]
            Optional. Allows a user who has previously enrolled in MFA to enroll with additional factors.<br />Note: Parameter can only be used with Universal Login; it cannot be used with Classic Login or custom MFA pages.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateGuardianEnrollmentTicketResponseContent
            Enrollment ticket successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.guardian.enrollments.create_ticket(
            user_id="user_id",
        )
        """
        _response = self._raw_client.create_ticket(
            user_id=user_id,
            email=email,
            send_mail=send_mail,
            email_locale=email_locale,
            factor=factor,
            allow_multiple_enrollments=allow_multiple_enrollments,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetGuardianEnrollmentResponseContent:
        """
        Retrieve details, such as status and type, for a specific multi-factor authentication enrollment registered to a user account.

        Parameters
        ----------
        id : str
            ID of the enrollment to be retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetGuardianEnrollmentResponseContent
            Enrollment successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.guardian.enrollments.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Remove a specific multi-factor authentication (MFA) enrollment from a user's account. This allows the user to re-enroll with MFA. For more information, review [Reset User Multi-Factor Authentication and Recovery Codes](https://auth0.com/docs/secure/multi-factor-authentication/reset-user-mfa).

        Parameters
        ----------
        id : str
            ID of the enrollment to be deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.guardian.enrollments.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data


class AsyncEnrollmentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEnrollmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawEnrollmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEnrollmentsClient
        """
        return self._raw_client

    async def create_ticket(
        self,
        *,
        user_id: str,
        email: typing.Optional[str] = OMIT,
        send_mail: typing.Optional[bool] = OMIT,
        email_locale: typing.Optional[str] = OMIT,
        factor: typing.Optional[GuardianEnrollmentFactorEnum] = OMIT,
        allow_multiple_enrollments: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateGuardianEnrollmentTicketResponseContent:
        """
        Create a [multi-factor authentication (MFA) enrollment ticket](https://auth0.com/docs/secure/multi-factor-authentication/auth0-guardian/create-custom-enrollment-tickets), and optionally send an email with the created ticket to a given user. Enrollment tickets can specify which factor users must enroll with or allow existing MFA users to enroll in additional factors.

        Parameters
        ----------
        user_id : str
            user_id for the enrollment ticket

        email : typing.Optional[str]
            alternate email to which the enrollment email will be sent. Optional - by default, the email will be sent to the user's default address

        send_mail : typing.Optional[bool]
            Send an email to the user to start the enrollment

        email_locale : typing.Optional[str]
            Optional. Specify the locale of the enrollment email. Used with send_email.

        factor : typing.Optional[GuardianEnrollmentFactorEnum]

        allow_multiple_enrollments : typing.Optional[bool]
            Optional. Allows a user who has previously enrolled in MFA to enroll with additional factors.<br />Note: Parameter can only be used with Universal Login; it cannot be used with Classic Login or custom MFA pages.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateGuardianEnrollmentTicketResponseContent
            Enrollment ticket successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.guardian.enrollments.create_ticket(
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_ticket(
            user_id=user_id,
            email=email,
            send_mail=send_mail,
            email_locale=email_locale,
            factor=factor,
            allow_multiple_enrollments=allow_multiple_enrollments,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetGuardianEnrollmentResponseContent:
        """
        Retrieve details, such as status and type, for a specific multi-factor authentication enrollment registered to a user account.

        Parameters
        ----------
        id : str
            ID of the enrollment to be retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetGuardianEnrollmentResponseContent
            Enrollment successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.guardian.enrollments.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Remove a specific multi-factor authentication (MFA) enrollment from a user's account. This allows the user to re-enroll with MFA. For more information, review [Reset User Multi-Factor Authentication and Recovery Codes](https://auth0.com/docs/secure/multi-factor-authentication/reset-user-mfa).

        Parameters
        ----------
        id : str
            ID of the enrollment to be deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.guardian.enrollments.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

ConnectionRegistrationEndpoint = str
"""
URL of the OPs Dynamic Client Registration Endpoint. RECOMMENDED but not REQUIRED. https://openid.net/specs/openid-connect-discovery-1_0.html#OpenID.Registration
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from .self_service_profile_branding_properties import SelfServiceProfileBrandingProperties

SelfServiceProfileBranding = typing.Optional[SelfServiceProfileBrandingProperties]



# This file was auto-generated by Fern from our API Definition.

import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.pagination import AsyncPager, SyncPager
from .....core.request_options import RequestOptions
from .....types.list_user_effective_permission_role_sources_response_content import (
    ListUserEffectivePermissionRoleSourcesResponseContent,
)
from .....types.user_effective_permission_role_source_response_content import (
    UserEffectivePermissionRoleSourceResponseContent,
)
from .raw_client import AsyncRawRolesClient, RawRolesClient


class RolesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawRolesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawRolesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawRolesClient
        """
        return self._raw_client

    def list(
        self,
        id: str,
        *,
        resource_server_identifier: str,
        permission_name: str,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[
        UserEffectivePermissionRoleSourceResponseContent, ListUserEffectivePermissionRoleSourcesResponseContent
    ]:
        """
        Lists the roles which grant the user a given permission, including roles assigned directly to the user and those inherited through group memberships.

        Parameters
        ----------
        id : str
            ID of the user to retrieve the permissions for.

        resource_server_identifier : str
            The identifier of the resource server for which to calculate user permissions.

        permission_name : str
            Name of this permission

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[UserEffectivePermissionRoleSourceResponseContent, ListUserEffectivePermissionRoleSourcesResponseContent]
            User's effective permission role sources successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.users.effective_permissions.sources.roles.list(
            id="id",
            from_="from",
            take=1,
            resource_server_identifier="resource_server_identifier",
            permission_name="permission_name",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            id,
            resource_server_identifier=resource_server_identifier,
            permission_name=permission_name,
            from_=from_,
            take=take,
            request_options=request_options,
        )


class AsyncRolesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawRolesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawRolesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawRolesClient
        """
        return self._raw_client

    async def list(
        self,
        id: str,
        *,
        resource_server_identifier: str,
        permission_name: str,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[
        UserEffectivePermissionRoleSourceResponseContent, ListUserEffectivePermissionRoleSourcesResponseContent
    ]:
        """
        Lists the roles which grant the user a given permission, including roles assigned directly to the user and those inherited through group memberships.

        Parameters
        ----------
        id : str
            ID of the user to retrieve the permissions for.

        resource_server_identifier : str
            The identifier of the resource server for which to calculate user permissions.

        permission_name : str
            Name of this permission

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[UserEffectivePermissionRoleSourceResponseContent, ListUserEffectivePermissionRoleSourcesResponseContent]
            User's effective permission role sources successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.users.effective_permissions.sources.roles.list(
                id="id",
                from_="from",
                take=1,
                resource_server_identifier="resource_server_identifier",
                permission_name="permission_name",
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            id,
            resource_server_identifier=resource_server_identifier,
            permission_name=permission_name,
            from_=from_,
            take=take,
            request_options=request_options,
        )



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class EventStreamCloudEventConnectionUpdatedObject0OptionsFederatedConnectionsAccessTokens(UniversalBaseModel):
    """
    Configuration for storing identity provider tokens in Auth0's Token Vault. When active, Auth0 securely stores access and refresh tokens from federated logins, enabling your application to make authenticated API calls on behalf of users.
    """

    active: bool = pydantic.Field()
    """
    Enables refresh tokens and access tokens collection for federated connections
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

FormFieldPaymentConfigChargeOneOffCurrencyEnum = typing.Union[
    typing.Literal["AUD", "CAD", "CHF", "EUR", "GBP", "INR", "MXN", "SEK", "USD"], typing.Any
]



from .conftest import get_client, verify_request_count


def test_clients_connections_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "clients.connections.get.0"
    client = get_client(test_id)
    client.clients.connections.get(
        id="id",
        strategy=["ad"],
        from_="from",
        take=1,
        fields="fields",
        include_fields=True,
    )
    verify_request_count(
        test_id,
        "GET",
        "/clients/id/connections",
        {"strategy": "ad", "from": "from", "take": "1", "fields": "fields", "include_fields": "true"},
        1,
    )



from .conftest import get_client, verify_request_count


def test_roles_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "roles.list_.0"
    client = get_client(test_id)
    client.roles.list(
        per_page=1,
        page=1,
        include_totals=True,
        name_filter="name_filter",
    )
    verify_request_count(
        test_id,
        "GET",
        "/roles",
        {"per_page": "1", "page": "1", "include_totals": "true", "name_filter": "name_filter"},
        1,
    )


def test_roles_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "roles.create.0"
    client = get_client(test_id)
    client.roles.create(
        name="name",
    )
    verify_request_count(test_id, "POST", "/roles", None, 1)


def test_roles_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "roles.get.0"
    client = get_client(test_id)
    client.roles.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/roles/id", None, 1)


def test_roles_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "roles.delete.0"
    client = get_client(test_id)
    client.roles.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/roles/id", None, 1)


def test_roles_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "roles.update.0"
    client = get_client(test_id)
    client.roles.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/roles/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.users_enrollment import UsersEnrollment
from .raw_client import AsyncRawEnrollmentsClient, RawEnrollmentsClient


class EnrollmentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEnrollmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawEnrollmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEnrollmentsClient
        """
        return self._raw_client

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> typing.List[UsersEnrollment]:
        """
        Retrieve the first [multi-factor authentication](https://auth0.com/docs/secure/multi-factor-authentication/multi-factor-authentication-factors) enrollment that a specific user has confirmed.

        Parameters
        ----------
        id : str
            ID of the user to list enrollments for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[UsersEnrollment]
            Enrollments successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.enrollments.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data


class AsyncEnrollmentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEnrollmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawEnrollmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEnrollmentsClient
        """
        return self._raw_client

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[UsersEnrollment]:
        """
        Retrieve the first [multi-factor authentication](https://auth0.com/docs/secure/multi-factor-authentication/multi-factor-authentication-factors) enrollment that a specific user has confirmed.

        Parameters
        ----------
        id : str
            ID of the user to list enrollments for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[UsersEnrollment]
            Enrollments successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.enrollments.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class LogLocationInfo(UniversalBaseModel):
    """
    Information about the location that triggered this event based on the `ip`.
    """

    country_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    Two-letter <a href="https://www.iso.org/iso-3166-country-codes.html">Alpha-2 ISO 3166-1</a> country code.
    """

    country_code_3: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="country_code3"),
        pydantic.Field(
            alias="country_code3",
            description='Three-letter <a href="https://www.iso.org/iso-3166-country-codes.html">Alpha-3 ISO 3166-1</a> country code.',
        ),
    ] = None
    """
    Three-letter <a href="https://www.iso.org/iso-3166-country-codes.html">Alpha-3 ISO 3166-1</a> country code.
    """

    country_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Full country name in English.
    """

    city_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Full city name in English.
    """

    latitude: typing.Optional[float] = pydantic.Field(default=None)
    """
    Global latitude (horizontal) position.
    """

    longitude: typing.Optional[float] = pydantic.Field(default=None)
    """
    Global longitude (vertical) position.
    """

    time_zone: typing.Optional[str] = pydantic.Field(default=None)
    """
    Time zone name as found in the <a href="https://www.iana.org/time-zones">tz database</a>.
    """

    continent_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    Continent the country is located within. Can be `AF` (Africa), `AN` (Antarctica), `AS` (Asia), `EU` (Europe), `NA` (North America), `OC` (Oceania) or `SA` (South America).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import encode_path_param
from ..core.pagination import AsyncPager, SyncPager
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.action import Action
from ..types.action_module_reference import ActionModuleReference
from ..types.action_secret_request import ActionSecretRequest
from ..types.action_trigger import ActionTrigger
from ..types.action_trigger_type_enum import ActionTriggerTypeEnum
from ..types.action_version_dependency import ActionVersionDependency
from ..types.create_action_response_content import CreateActionResponseContent
from ..types.deploy_action_response_content import DeployActionResponseContent
from ..types.get_action_response_content import GetActionResponseContent
from ..types.list_actions_paginated_response_content import ListActionsPaginatedResponseContent
from ..types.test_action_payload import TestActionPayload
from ..types.test_action_response_content import TestActionResponseContent
from ..types.update_action_response_content import UpdateActionResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawActionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        trigger_id: typing.Optional[ActionTriggerTypeEnum] = None,
        action_name: typing.Optional[str] = None,
        deployed: typing.Optional[bool] = None,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        installed: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Action, ListActionsPaginatedResponseContent]:
        """
        Retrieve all actions.

        Parameters
        ----------
        trigger_id : typing.Optional[ActionTriggerTypeEnum]
            An actions extensibility point.

        action_name : typing.Optional[str]
            The name of the action to retrieve.

        deployed : typing.Optional[bool]
            Optional filter to only retrieve actions that are deployed.

        page : typing.Optional[int]
            Use this field to request a specific page of the list results.

        per_page : typing.Optional[int]
            The maximum number of results to be returned by the server in single response. 20 by default

        installed : typing.Optional[bool]
            Optional. When true, return only installed actions. When false, return only custom actions. Returns all actions by default.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Action, ListActionsPaginatedResponseContent]
            The actions were retrieved.
        """
        page = page if page is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            "actions/actions",
            method="GET",
            params={
                "triggerId": trigger_id,
                "actionName": action_name,
                "deployed": deployed,
                "page": page,
                "per_page": per_page,
                "installed": installed,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListActionsPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListActionsPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.actions
                _has_next = len(_items or []) > 0
                _get_next = lambda: self.list(
                    trigger_id=trigger_id,
                    action_name=action_name,
                    deployed=deployed,
                    page=page + 1,
                    per_page=per_page,
                    installed=installed,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        name: str,
        supported_triggers: typing.Sequence[ActionTrigger],
        code: typing.Optional[str] = OMIT,
        dependencies: typing.Optional[typing.Sequence[ActionVersionDependency]] = OMIT,
        runtime: typing.Optional[str] = OMIT,
        secrets: typing.Optional[typing.Sequence[ActionSecretRequest]] = OMIT,
        modules: typing.Optional[typing.Sequence[ActionModuleReference]] = OMIT,
        deploy: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateActionResponseContent]:
        """
        Create an action. Once an action is created, it must be deployed, and then bound to a trigger before it will be executed as part of a flow.

        Parameters
        ----------
        name : str
            The name of an action.

        supported_triggers : typing.Sequence[ActionTrigger]
            The list of triggers that this action supports. At this time, an action can only target a single trigger at a time.

        code : typing.Optional[str]
            The source code of the action.

        dependencies : typing.Optional[typing.Sequence[ActionVersionDependency]]
            The list of third party npm modules, and their versions, that this action depends on.

        runtime : typing.Optional[str]
            The Node runtime. For example: `node22`, defaults to `node22`

        secrets : typing.Optional[typing.Sequence[ActionSecretRequest]]
            The list of secrets that are included in an action or a version of an action.

        modules : typing.Optional[typing.Sequence[ActionModuleReference]]
            The list of action modules and their versions used by this action.

        deploy : typing.Optional[bool]
            True if the action should be deployed after creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateActionResponseContent]
            Action successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "actions/actions",
            method="POST",
            json={
                "name": name,
                "supported_triggers": convert_and_respect_annotation_metadata(
                    object_=supported_triggers, annotation=typing.Sequence[ActionTrigger], direction="write"
                ),
                "code": code,
                "dependencies": convert_and_respect_annotation_metadata(
                    object_=dependencies, annotation=typing.Sequence[ActionVersionDependency], direction="write"
                ),
                "runtime": runtime,
                "secrets": convert_and_respect_annotation_metadata(
                    object_=secrets, annotation=typing.Sequence[ActionSecretRequest], direction="write"
                ),
                "modules": convert_and_respect_annotation_metadata(
                    object_=modules, annotation=typing.Sequence[ActionModuleReference], direction="write"
                ),
                "deploy": deploy,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateActionResponseContent,
                    parse_obj_as(
                        type_=CreateActionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetActionResponseContent]:
        """
        Retrieve an action by its ID.

        Parameters
        ----------
        id : str
            The ID of the action to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetActionResponseContent]
            The action was retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"actions/actions/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetActionResponseContent,
                    parse_obj_as(
                        type_=GetActionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, id: str, *, force: typing.Optional[bool] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Deletes an action and all of its associated versions. An action must be unbound from all triggers before it can be deleted.

        Parameters
        ----------
        id : str
            The ID of the action to delete.

        force : typing.Optional[bool]
            Force action deletion detaching bindings

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"actions/actions/{encode_path_param(id)}",
            method="DELETE",
            params={
                "force": force,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        supported_triggers: typing.Optional[typing.Sequence[ActionTrigger]] = OMIT,
        code: typing.Optional[str] = OMIT,
        dependencies: typing.Optional[typing.Sequence[ActionVersionDependency]] = OMIT,
        runtime: typing.Optional[str] = OMIT,
        secrets: typing.Optional[typing.Sequence[ActionSecretRequest]] = OMIT,
        modules: typing.Optional[typing.Sequence[ActionModuleReference]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateActionResponseContent]:
        """
        Update an existing action. If this action is currently bound to a trigger, updating it will **not** affect any user flows until the action is deployed.

        Parameters
        ----------
        id : str
            The id of the action to update.

        name : typing.Optional[str]
            The name of an action.

        supported_triggers : typing.Optional[typing.Sequence[ActionTrigger]]
            The list of triggers that this action supports. At this time, an action can only target a single trigger at a time.

        code : typing.Optional[str]
            The source code of the action.

        dependencies : typing.Optional[typing.Sequence[ActionVersionDependency]]
            The list of third party npm modules, and their versions, that this action depends on.

        runtime : typing.Optional[str]
            The Node runtime. For example: `node22`, defaults to `node22`

        secrets : typing.Optional[typing.Sequence[ActionSecretRequest]]
            The list of secrets that are included in an action or a version of an action.

        modules : typing.Optional[typing.Sequence[ActionModuleReference]]
            The list of action modules and their versions used by this action.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateActionResponseContent]
            Action successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"actions/actions/{encode_path_param(id)}",
            method="PATCH",
            json={
                "name": name,
                "supported_triggers": convert_and_respect_annotation_metadata(
                    object_=supported_triggers, annotation=typing.Sequence[ActionTrigger], direction="write"
                ),
                "code": code,
                "dependencies": convert_and_respect_annotation_metadata(
                    object_=dependencies, annotation=typing.Sequence[ActionVersionDependency], direction="write"
                ),
                "runtime": runtime,
                "secrets": convert_and_respect_annotation_metadata(
                    object_=secrets, annotation=typing.Sequence[ActionSecretRequest], direction="write"
                ),
                "modules": convert_and_respect_annotation_metadata(
                    object_=modules, annotation=typing.Sequence[ActionModuleReference], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateActionResponseContent,
                    parse_obj_as(
                        type_=UpdateActionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def deploy(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeployActionResponseContent]:
        """
        Deploy an action. Deploying an action will create a new immutable version of the action. If the action is currently bound to a trigger, then the system will begin executing the newly deployed version of the action immediately. Otherwise, the action will only be executed as a part of a flow once it is bound to that flow.

        Parameters
        ----------
        id : str
            The ID of an action.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeployActionResponseContent]
            Request to create action version was accepted.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"actions/actions/{encode_path_param(id)}/deploy",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeployActionResponseContent,
                    parse_obj_as(
                        type_=DeployActionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def test(
        self, id: str, *, payload: TestActionPayload, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[TestActionResponseContent]:
        """
        Test an action. After updating an action, it can be tested prior to being deployed to ensure it behaves as expected.

        Parameters
        ----------
        id : str
            The id of the action to test.

        payload : TestActionPayload

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TestActionResponseContent]
            Test action version successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"actions/actions/{encode_path_param(id)}/test",
            method="POST",
            json={
                "payload": payload,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TestActionResponseContent,
                    parse_obj_as(
                        type_=TestActionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawActionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        trigger_id: typing.Optional[ActionTriggerTypeEnum] = None,
        action_name: typing.Optional[str] = None,
        deployed: typing.Optional[bool] = None,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        installed: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Action, ListActionsPaginatedResponseContent]:
        """
        Retrieve all actions.

        Parameters
        ----------
        trigger_id : typing.Optional[ActionTriggerTypeEnum]
            An actions extensibility point.

        action_name : typing.Optional[str]
            The name of the action to retrieve.

        deployed : typing.Optional[bool]
            Optional filter to only retrieve actions that are deployed.

        page : typing.Optional[int]
            Use this field to request a specific page of the list results.

        per_page : typing.Optional[int]
            The maximum number of results to be returned by the server in single response. 20 by default

        installed : typing.Optional[bool]
            Optional. When true, return only installed actions. When false, return only custom actions. Returns all actions by default.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Action, ListActionsPaginatedResponseContent]
            The actions were retrieved.
        """
        page = page if page is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            "actions/actions",
            method="GET",
            params={
                "triggerId": trigger_id,
                "actionName": action_name,
                "deployed": deployed,
                "page": page,
                "per_page": per_page,
                "installed": installed,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListActionsPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListActionsPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.actions
                _has_next = len(_items or []) > 0

                async def _get_next():
                    return await self.list(
                        trigger_id=trigger_id,
                        action_name=action_name,
                        deployed=deployed,
                        page=page + 1,
                        per_page=per_page,
                        installed=installed,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        name: str,
        supported_triggers: typing.Sequence[ActionTrigger],
        code: typing.Optional[str] = OMIT,
        dependencies: typing.Optional[typing.Sequence[ActionVersionDependency]] = OMIT,
        runtime: typing.Optional[str] = OMIT,
        secrets: typing.Optional[typing.Sequence[ActionSecretRequest]] = OMIT,
        modules: typing.Optional[typing.Sequence[ActionModuleReference]] = OMIT,
        deploy: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateActionResponseContent]:
        """
        Create an action. Once an action is created, it must be deployed, and then bound to a trigger before it will be executed as part of a flow.

        Parameters
        ----------
        name : str
            The name of an action.

        supported_triggers : typing.Sequence[ActionTrigger]
            The list of triggers that this action supports. At this time, an action can only target a single trigger at a time.

        code : typing.Optional[str]
            The source code of the action.

        dependencies : typing.Optional[typing.Sequence[ActionVersionDependency]]
            The list of third party npm modules, and their versions, that this action depends on.

        runtime : typing.Optional[str]
            The Node runtime. For example: `node22`, defaults to `node22`

        secrets : typing.Optional[typing.Sequence[ActionSecretRequest]]
            The list of secrets that are included in an action or a version of an action.

        modules : typing.Optional[typing.Sequence[ActionModuleReference]]
            The list of action modules and their versions used by this action.

        deploy : typing.Optional[bool]
            True if the action should be deployed after creation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateActionResponseContent]
            Action successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "actions/actions",
            method="POST",
            json={
                "name": name,
                "supported_triggers": convert_and_respect_annotation_metadata(
                    object_=supported_triggers, annotation=typing.Sequence[ActionTrigger], direction="write"
                ),
                "code": code,
                "dependencies": convert_and_respect_annotation_metadata(
                    object_=dependencies, annotation=typing.Sequence[ActionVersionDependency], direction="write"
                ),
                "runtime": runtime,
                "secrets": convert_and_respect_annotation_metadata(
                    object_=secrets, annotation=typing.Sequence[ActionSecretRequest], direction="write"
                ),
                "modules": convert_and_respect_annotation_metadata(
                    object_=modules, annotation=typing.Sequence[ActionModuleReference], direction="write"
                ),
                "deploy": deploy,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateActionResponseContent,
                    parse_obj_as(
                        type_=CreateActionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetActionResponseContent]:
        """
        Retrieve an action by its ID.

        Parameters
        ----------
        id : str
            The ID of the action to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetActionResponseContent]
            The action was retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"actions/actions/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetActionResponseContent,
                    parse_obj_as(
                        type_=GetActionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, force: typing.Optional[bool] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Deletes an action and all of its associated versions. An action must be unbound from all triggers before it can be deleted.

        Parameters
        ----------
        id : str
            The ID of the action to delete.

        force : typing.Optional[bool]
            Force action deletion detaching bindings

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"actions/actions/{encode_path_param(id)}",
            method="DELETE",
            params={
                "force": force,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        supported_triggers: typing.Optional[typing.Sequence[ActionTrigger]] = OMIT,
        code: typing.Optional[str] = OMIT,
        dependencies: typing.Optional[typing.Sequence[ActionVersionDependency]] = OMIT,
        runtime: typing.Optional[str] = OMIT,
        secrets: typing.Optional[typing.Sequence[ActionSecretRequest]] = OMIT,
        modules: typing.Optional[typing.Sequence[ActionModuleReference]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateActionResponseContent]:
        """
        Update an existing action. If this action is currently bound to a trigger, updating it will **not** affect any user flows until the action is deployed.

        Parameters
        ----------
        id : str
            The id of the action to update.

        name : typing.Optional[str]
            The name of an action.

        supported_triggers : typing.Optional[typing.Sequence[ActionTrigger]]
            The list of triggers that this action supports. At this time, an action can only target a single trigger at a time.

        code : typing.Optional[str]
            The source code of the action.

        dependencies : typing.Optional[typing.Sequence[ActionVersionDependency]]
            The list of third party npm modules, and their versions, that this action depends on.

        runtime : typing.Optional[str]
            The Node runtime. For example: `node22`, defaults to `node22`

        secrets : typing.Optional[typing.Sequence[ActionSecretRequest]]
            The list of secrets that are included in an action or a version of an action.

        modules : typing.Optional[typing.Sequence[ActionModuleReference]]
            The list of action modules and their versions used by this action.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateActionResponseContent]
            Action successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"actions/actions/{encode_path_param(id)}",
            method="PATCH",
            json={
                "name": name,
                "supported_triggers": convert_and_respect_annotation_metadata(
                    object_=supported_triggers, annotation=typing.Sequence[ActionTrigger], direction="write"
                ),
                "code": code,
                "dependencies": convert_and_respect_annotation_metadata(
                    object_=dependencies, annotation=typing.Sequence[ActionVersionDependency], direction="write"
                ),
                "runtime": runtime,
                "secrets": convert_and_respect_annotation_metadata(
                    object_=secrets, annotation=typing.Sequence[ActionSecretRequest], direction="write"
                ),
                "modules": convert_and_respect_annotation_metadata(
                    object_=modules, annotation=typing.Sequence[ActionModuleReference], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateActionResponseContent,
                    parse_obj_as(
                        type_=UpdateActionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def deploy(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeployActionResponseContent]:
        """
        Deploy an action. Deploying an action will create a new immutable version of the action. If the action is currently bound to a trigger, then the system will begin executing the newly deployed version of the action immediately. Otherwise, the action will only be executed as a part of a flow once it is bound to that flow.

        Parameters
        ----------
        id : str
            The ID of an action.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeployActionResponseContent]
            Request to create action version was accepted.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"actions/actions/{encode_path_param(id)}/deploy",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeployActionResponseContent,
                    parse_obj_as(
                        type_=DeployActionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def test(
        self, id: str, *, payload: TestActionPayload, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[TestActionResponseContent]:
        """
        Test an action. After updating an action, it can be tested prior to being deployed to ensure it behaves as expected.

        Parameters
        ----------
        id : str
            The id of the action to test.

        payload : TestActionPayload

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TestActionResponseContent]
            Test action version successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"actions/actions/{encode_path_param(id)}/test",
            method="POST",
            json={
                "payload": payload,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TestActionResponseContent,
                    parse_obj_as(
                        type_=TestActionResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventGroupMemberAddedObjectMember1MemberTypeEnum = typing.Union[
    typing.Literal["connection"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .action import Action


class ListActionsPaginatedResponseContent(UniversalBaseModel):
    total: typing.Optional[float] = pydantic.Field(default=None)
    """
    The total result count.
    """

    page: typing.Optional[float] = pydantic.Field(default=None)
    """
    Page index of the results being returned. First page is 0.
    """

    per_page: typing.Optional[float] = pydantic.Field(default=None)
    """
    Number of results per page.
    """

    actions: typing.Optional[typing.List[Action]] = pydantic.Field(default=None)
    """
    The list of actions.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_json_create_json import FlowActionJsonCreateJson
from .flow_action_json_parse_json import FlowActionJsonParseJson
from .flow_action_json_serialize_json import FlowActionJsonSerializeJson

FlowActionJson = typing.Union[FlowActionJsonCreateJson, FlowActionJsonParseJson, FlowActionJsonSerializeJson]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .self_service_profile_sso_ticket_google_workspace_config import SelfServiceProfileSsoTicketGoogleWorkspaceConfig
from .self_service_profile_sso_ticket_provisioning_scope_enum import SelfServiceProfileSsoTicketProvisioningScopeEnum


class SelfServiceProfileSsoTicketProvisioningConfig(UniversalBaseModel):
    """
    Configuration for the setup of Provisioning in the self-service flow.
    """

    scopes: typing.Optional[typing.List[SelfServiceProfileSsoTicketProvisioningScopeEnum]] = pydantic.Field(
        default=None
    )
    """
    The scopes of the SCIM tokens generated during the self-service flow.
    """

    google_workspace: typing.Optional[SelfServiceProfileSsoTicketGoogleWorkspaceConfig] = None
    token_lifetime: typing.Optional[int] = pydantic.Field(default=None)
    """
    Lifetime of the tokens in seconds. Must be greater than 900. If not provided, the tokens don't expire.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_organizations_clientGrants_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "organizations.client_grants.list_.0"
    client = get_client(test_id)
    client.organizations.client_grants.list(
        id="id",
        audience="audience",
        client_id="client_id",
        grant_ids=["grant_ids"],
        page=1,
        per_page=1,
        include_totals=True,
    )
    verify_request_count(
        test_id,
        "GET",
        "/organizations/id/client-grants",
        {
            "audience": "audience",
            "client_id": "client_id",
            "grant_ids": "grant_ids",
            "page": "1",
            "per_page": "1",
            "include_totals": "true",
        },
        1,
    )


def test_organizations_clientGrants_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "organizations.client_grants.create.0"
    client = get_client(test_id)
    client.organizations.client_grants.create(
        id="id",
        grant_id="grant_id",
    )
    verify_request_count(test_id, "POST", "/organizations/id/client-grants", None, 1)


def test_organizations_clientGrants_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "organizations.client_grants.delete.0"
    client = get_client(test_id)
    client.organizations.client_grants.delete(
        id="id",
        grant_id="grant_id",
    )
    verify_request_count(test_id, "DELETE", "/organizations/id/client-grants/grant_id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class CimdValidationResult(UniversalBaseModel):
    """
    Validation result for the Client ID Metadata Document
    """

    valid: bool = pydantic.Field()
    """
    Whether the metadata document passed validation
    """

    violations: typing.List[str] = pydantic.Field()
    """
    Array of validation violation messages (if any)
    """

    warnings: typing.List[str] = pydantic.Field()
    """
    Array of warning messages (if any)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .connection_domain_aliases_items_one import ConnectionDomainAliasesItemsOne

ConnectionDomainAliases = typing.List[ConnectionDomainAliasesItemsOne]
"""
Email domains associated with this connection for Home Realm Discovery (HRD). When a user's email matches one of these domains, they are automatically routed to this connection during authentication.
"""



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawEmailsClient, RawEmailsClient

if typing.TYPE_CHECKING:
    from .provider.client import AsyncProviderClient, ProviderClient


class EmailsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEmailsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._provider: typing.Optional[ProviderClient] = None

    @property
    def with_raw_response(self) -> RawEmailsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEmailsClient
        """
        return self._raw_client

    @property
    def provider(self):
        if self._provider is None:
            from .provider.client import ProviderClient  # noqa: E402

            self._provider = ProviderClient(client_wrapper=self._client_wrapper)
        return self._provider


class AsyncEmailsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEmailsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._provider: typing.Optional[AsyncProviderClient] = None

    @property
    def with_raw_response(self) -> AsyncRawEmailsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEmailsClient
        """
        return self._raw_client

    @property
    def provider(self):
        if self._provider is None:
            from .provider.client import AsyncProviderClient  # noqa: E402

            self._provider = AsyncProviderClient(client_wrapper=self._client_wrapper)
        return self._provider



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .client_grant_default_for_enum import ClientGrantDefaultForEnum
from .client_grant_organization_usage_enum import ClientGrantOrganizationUsageEnum
from .client_grant_subject_type_enum import ClientGrantSubjectTypeEnum


class CreateClientGrantResponseContent(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the client grant.
    """

    client_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the client.
    """

    audience: typing.Optional[str] = pydantic.Field(default=None)
    """
    The audience (API identifier) of this client grant.
    """

    scope: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Scopes allowed for this client grant.
    """

    organization_usage: typing.Optional[ClientGrantOrganizationUsageEnum] = None
    allow_any_organization: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If enabled, any organization can be used with this grant. If disabled (default), the grant must be explicitly assigned to the desired organizations.
    """

    default_for: typing.Optional[ClientGrantDefaultForEnum] = None
    is_system: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If enabled, this grant is a special grant created by Auth0. It cannot be modified or deleted directly.
    """

    subject_type: typing.Optional[ClientGrantSubjectTypeEnum] = None
    authorization_details_types: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Types of authorization_details allowed for this client grant.
    """

    allow_all_scopes: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If enabled, all scopes configured on the resource server are allowed for this grant.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from .connection_auth_params_map import ConnectionAuthParamsMap
from .connection_auth_params_o_auth_2 import ConnectionAuthParamsOAuth2
from .connection_authorization_endpoint import ConnectionAuthorizationEndpoint
from .connection_client_id_o_auth_2 import ConnectionClientIdOAuth2
from .connection_client_secret_o_auth_2 import ConnectionClientSecretOAuth2
from .connection_custom_headers_o_auth_2 import ConnectionCustomHeadersOAuth2
from .connection_end_session_endpoint import ConnectionEndSessionEndpoint
from .connection_fields_map import ConnectionFieldsMap
from .connection_icon_url import ConnectionIconUrl
from .connection_options_common import ConnectionOptionsCommon
from .connection_scope_o_auth_2 import ConnectionScopeOAuth2
from .connection_scripts_o_auth_2 import ConnectionScriptsOAuth2
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum
from .connection_token_endpoint import ConnectionTokenEndpoint
from .connection_upstream_params import ConnectionUpstreamParams


class ConnectionOptionsOAuth2(ConnectionOptionsCommon):
    """
    Options for the 'oauth2' connection
    """

    auth_params: typing_extensions.Annotated[
        typing.Optional[ConnectionAuthParamsOAuth2],
        FieldMetadata(alias="authParams"),
        pydantic.Field(alias="authParams"),
    ] = None
    auth_params_map: typing_extensions.Annotated[
        typing.Optional[ConnectionAuthParamsMap],
        FieldMetadata(alias="authParamsMap"),
        pydantic.Field(alias="authParamsMap"),
    ] = None
    authorization_url: typing_extensions.Annotated[
        typing.Optional[ConnectionAuthorizationEndpoint],
        FieldMetadata(alias="authorizationURL"),
        pydantic.Field(alias="authorizationURL"),
    ] = None
    client_id: typing.Optional[ConnectionClientIdOAuth2] = None
    client_secret: typing.Optional[ConnectionClientSecretOAuth2] = None
    custom_headers: typing_extensions.Annotated[
        typing.Optional[ConnectionCustomHeadersOAuth2],
        FieldMetadata(alias="customHeaders"),
        pydantic.Field(alias="customHeaders"),
    ] = None
    fields_map: typing_extensions.Annotated[
        typing.Optional[ConnectionFieldsMap], FieldMetadata(alias="fieldsMap"), pydantic.Field(alias="fieldsMap")
    ] = None
    icon_url: typing.Optional[ConnectionIconUrl] = None
    logout_url: typing_extensions.Annotated[
        typing.Optional[ConnectionEndSessionEndpoint],
        FieldMetadata(alias="logoutUrl"),
        pydantic.Field(alias="logoutUrl"),
    ] = None
    pkce_enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When true, enables Proof Key for Code Exchange (PKCE) for the authorization code flow. PKCE provides additional security by preventing authorization code interception attacks.
    """

    scope: typing.Optional[ConnectionScopeOAuth2] = None
    scripts: typing.Optional[ConnectionScriptsOAuth2] = None
    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    token_url: typing_extensions.Annotated[
        typing.Optional[ConnectionTokenEndpoint], FieldMetadata(alias="tokenURL"), pydantic.Field(alias="tokenURL")
    ] = None
    upstream_params: typing.Optional[ConnectionUpstreamParams] = None
    use_oauth_spec_scope: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="useOauthSpecScope"),
        pydantic.Field(
            alias="useOauthSpecScope",
            description="When true, uses space-delimited scopes (per OAuth 2.0 spec) instead of comma-delimited when calling the identity provider's authorization endpoint. Only relevant when using the connection_scope parameter. See https://auth0.com/docs/authenticate/identity-providers/adding-scopes-for-an-external-idp#pass-scopes-to-authorize-endpoint",
        ),
    ] = None
    """
    When true, uses space-delimited scopes (per OAuth 2.0 spec) instead of comma-delimited when calling the identity provider's authorization endpoint. Only relevant when using the connection_scope parameter. See https://auth0.com/docs/authenticate/identity-providers/adding-scopes-for-an-external-idp#pass-scopes-to-authorize-endpoint
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .resource_server_authorization_policy import ResourceServerAuthorizationPolicy
from .resource_server_consent_policy_enum import ResourceServerConsentPolicyEnum
from .resource_server_proof_of_possession import ResourceServerProofOfPossession
from .resource_server_scope import ResourceServerScope
from .resource_server_subject_type_authorization import ResourceServerSubjectTypeAuthorization
from .resource_server_token_dialect_response_enum import ResourceServerTokenDialectResponseEnum
from .resource_server_token_encryption import ResourceServerTokenEncryption
from .signing_algorithm_enum import SigningAlgorithmEnum


class ResourceServer(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the API (resource server).
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Friendly name for this resource server. Can not contain `<` or `>` characters.
    """

    is_system: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this is an Auth0 system API (true) or a custom API (false).
    """

    identifier: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the API used as the audience parameter on authorization calls. Can not be changed once set.
    """

    scopes: typing.Optional[typing.List[ResourceServerScope]] = pydantic.Field(default=None)
    """
    List of permissions (scopes) that this API uses.
    """

    signing_alg: typing.Optional[SigningAlgorithmEnum] = None
    signing_secret: typing.Optional[str] = pydantic.Field(default=None)
    """
    Secret used to sign tokens when using symmetric algorithms (HS256).
    """

    allow_offline_access: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether refresh tokens can be issued for this API (true) or not (false).
    """

    allow_online_access: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether Online Refresh Tokens can be issued for this API (true) or not (false).
    """

    allow_online_access_with_ephemeral_sessions: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether Online Refresh Tokens can be issued even when sessions are configured as ephemeral (true) or not (false).
    """

    skip_consent_for_verifiable_first_party_clients: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether to skip user consent for applications flagged as first party (true) or not (false).
    """

    token_lifetime: typing.Optional[int] = pydantic.Field(default=None)
    """
    Expiration value (in seconds) for access tokens issued for this API from the token endpoint.
    """

    token_lifetime_for_web: typing.Optional[int] = pydantic.Field(default=None)
    """
    Expiration value (in seconds) for access tokens issued for this API via Implicit or Hybrid Flows. Cannot be greater than the `token_lifetime` value.
    """

    enforce_policies: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether authorization polices are enforced (true) or unenforced (false).
    """

    token_dialect: typing.Optional[ResourceServerTokenDialectResponseEnum] = None
    token_encryption: typing.Optional[ResourceServerTokenEncryption] = None
    consent_policy: typing.Optional[ResourceServerConsentPolicyEnum] = None
    authorization_details: typing.Optional[typing.List[typing.Any]] = None
    proof_of_possession: typing.Optional[ResourceServerProofOfPossession] = None
    subject_type_authorization: typing.Optional[ResourceServerSubjectTypeAuthorization] = None
    authorization_policy: typing.Optional[ResourceServerAuthorizationPolicy] = None
    client_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The client ID of the client that this resource server is linked to
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .token_quota_configuration import TokenQuotaConfiguration


class DefaultTokenQuota(UniversalBaseModel):
    """
    Token Quota configuration, to configure quotas for token issuance for clients and organizations. Applied to all clients and organizations unless overridden in individual client or organization settings.
    """

    clients: typing.Optional[TokenQuotaConfiguration] = None
    organizations: typing.Optional[TokenQuotaConfiguration] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .authentication_method_type_enum import AuthenticationMethodTypeEnum
from .preferred_authentication_method_enum import PreferredAuthenticationMethodEnum
from .user_authentication_method_properties import UserAuthenticationMethodProperties


class GetUserAuthenticationMethodResponseContent(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    The ID of the authentication method (auto generated)
    """

    type: AuthenticationMethodTypeEnum
    confirmed: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The authentication method status
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    A human-readable label to identify the authentication method
    """

    authentication_methods: typing.Optional[typing.List[UserAuthenticationMethodProperties]] = None
    preferred_authentication_method: typing.Optional[PreferredAuthenticationMethodEnum] = None
    link_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of a linked authentication method. Linked authentication methods will be deleted together.
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to phone authentication methods only. The destination phone number used to send verification codes via text and voice.
    """

    email: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to email and email-verification authentication methods only. The email address used to send verification messages.
    """

    key_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to webauthn authentication methods only. The ID of the generated credential.
    """

    public_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to webauthn authentication methods only. The public key.
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Authenticator creation date
    """

    enrolled_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Enrollment date
    """

    last_auth_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Last authentication
    """

    credential_device_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to passkeys only. The kind of device the credential is stored on as defined by backup eligibility. "single_device" credentials cannot be backed up and synced to another device, "multi_device" credentials can be backed up if enabled by the end-user.
    """

    credential_backed_up: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Applies to passkeys only. Whether the credential was backed up.
    """

    identity_user_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to passkeys only. The ID of the user identity linked with the authentication method.
    """

    user_agent: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to passkeys only. The user-agent of the browser used to create the passkey.
    """

    user_handle: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to passkeys only. The user handle of the user identity.
    """

    transports: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Applies to passkeys only. The transports used by clients to communicate with the authenticator.
    """

    aaguid: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to passkey authentication methods only. Authenticator Attestation Globally Unique Identifier.
    """

    relying_party_identifier: typing.Optional[str] = pydantic.Field(default=None)
    """
    Applies to webauthn/passkey authentication methods only. The credential's relying party identifier.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .delete_user_identity_response_content_item import DeleteUserIdentityResponseContentItem

DeleteUserIdentityResponseContent = typing.List[DeleteUserIdentityResponseContentItem]
"""
An array of objects with information about the user's identities.
"""



from .conftest import get_client, verify_request_count


def test_jobs_verificationEmail_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "jobs.verification_email.create.0"
    client = get_client(test_id)
    client.jobs.verification_email.create(
        user_id="user_id",
    )
    verify_request_count(test_id, "POST", "/jobs/verification-email", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .connection_authentication_purpose import ConnectionAuthenticationPurpose
from .connection_connected_accounts_purpose import ConnectionConnectedAccountsPurpose
from .connection_options import ConnectionOptions
from .connections_metadata import ConnectionsMetadata
from .cross_app_access_requesting_app import CrossAppAccessRequestingApp
from .cross_app_access_resource_app import CrossAppAccessResourceApp


class CreateConnectionResponseContent(UniversalBaseModel):
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the connection
    """

    display_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Connection name used in login screen
    """

    options: typing.Optional[ConnectionOptions] = None
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The connection's identifier
    """

    strategy: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of the connection, related to the identity provider
    """

    realms: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Defines the realms for which the connection will be used (ie: email domains). If the array is empty or the property is not specified, the connection name will be added as realm.
    """

    enabled_clients: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    DEPRECATED property. Use the GET /connections/:id/clients endpoint to get the ids of the clients for which the connection is enabled
    """

    is_domain_connection: typing.Optional[bool] = pydantic.Field(default=None)
    """
    True if the connection is domain level
    """

    show_as_button: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables showing a button for the connection in the login page (new experience only). If false, it will be usable only by HRD.
    """

    metadata: typing.Optional[ConnectionsMetadata] = None
    authentication: typing.Optional[ConnectionAuthenticationPurpose] = None
    connected_accounts: typing.Optional[ConnectionConnectedAccountsPurpose] = None
    cross_app_access_requesting_app: typing.Optional[CrossAppAccessRequestingApp] = None
    cross_app_access_resource_app: typing.Optional[CrossAppAccessResourceApp] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from .raw_client import AsyncRawAuthenticatorsClient, RawAuthenticatorsClient


class AuthenticatorsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAuthenticatorsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAuthenticatorsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAuthenticatorsClient
        """
        return self._raw_client

    def delete_all(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Remove all authenticators registered to a given user ID, such as OTP, email, phone, and push-notification. This action cannot be undone. For more information, review [Manage Authentication Methods with Management API](https://auth0.com/docs/secure/multi-factor-authentication/manage-mfa-auth0-apis/manage-authentication-methods-with-management-api).

        Parameters
        ----------
        id : str
            ID of the user to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.authenticators.delete_all(
            id="id",
        )
        """
        _response = self._raw_client.delete_all(id, request_options=request_options)
        return _response.data


class AsyncAuthenticatorsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAuthenticatorsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAuthenticatorsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAuthenticatorsClient
        """
        return self._raw_client

    async def delete_all(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Remove all authenticators registered to a given user ID, such as OTP, email, phone, and push-notification. This action cannot be undone. For more information, review [Manage Authentication Methods with Management API](https://auth0.com/docs/secure/multi-factor-authentication/manage-mfa-auth0-apis/manage-authentication-methods-with-management-api).

        Parameters
        ----------
        id : str
            ID of the user to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.authenticators.delete_all(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_all(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .synchronized_group_payload import SynchronizedGroupPayload


class ListSynchronizedGroupsResponseContent(UniversalBaseModel):
    groups: typing.List[SynchronizedGroupPayload] = pydantic.Field()
    """
    Array of Google Workspace group ids configured for synchronization.
    """

    next: typing.Optional[str] = pydantic.Field(default=None)
    """
    The cursor to be used as the "from" query parameter for the next page of results.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_guardian_factors_pushNotification_get_apns_provider() -> None:
    """Test getApnsProvider endpoint with WireMock"""
    test_id = "guardian.factors.push_notification.get_apns_provider.0"
    client = get_client(test_id)
    client.guardian.factors.push_notification.get_apns_provider()
    verify_request_count(test_id, "GET", "/guardian/factors/push-notification/providers/apns", None, 1)


def test_guardian_factors_pushNotification_set_apns_provider() -> None:
    """Test setApnsProvider endpoint with WireMock"""
    test_id = "guardian.factors.push_notification.set_apns_provider.0"
    client = get_client(test_id)
    client.guardian.factors.push_notification.set_apns_provider()
    verify_request_count(test_id, "PUT", "/guardian/factors/push-notification/providers/apns", None, 1)


def test_guardian_factors_pushNotification_update_apns_provider() -> None:
    """Test updateApnsProvider endpoint with WireMock"""
    test_id = "guardian.factors.push_notification.update_apns_provider.0"
    client = get_client(test_id)
    client.guardian.factors.push_notification.update_apns_provider()
    verify_request_count(test_id, "PATCH", "/guardian/factors/push-notification/providers/apns", None, 1)


def test_guardian_factors_pushNotification_set_fcm_provider() -> None:
    """Test setFcmProvider endpoint with WireMock"""
    test_id = "guardian.factors.push_notification.set_fcm_provider.0"
    client = get_client(test_id)
    client.guardian.factors.push_notification.set_fcm_provider()
    verify_request_count(test_id, "PUT", "/guardian/factors/push-notification/providers/fcm", None, 1)


def test_guardian_factors_pushNotification_update_fcm_provider() -> None:
    """Test updateFcmProvider endpoint with WireMock"""
    test_id = "guardian.factors.push_notification.update_fcm_provider.0"
    client = get_client(test_id)
    client.guardian.factors.push_notification.update_fcm_provider()
    verify_request_count(test_id, "PATCH", "/guardian/factors/push-notification/providers/fcm", None, 1)


def test_guardian_factors_pushNotification_set_fcmv_1_provider() -> None:
    """Test setFcmv1Provider endpoint with WireMock"""
    test_id = "guardian.factors.push_notification.set_fcmv_1_provider.0"
    client = get_client(test_id)
    client.guardian.factors.push_notification.set_fcmv_1_provider()
    verify_request_count(test_id, "PUT", "/guardian/factors/push-notification/providers/fcmv1", None, 1)


def test_guardian_factors_pushNotification_update_fcmv_1_provider() -> None:
    """Test updateFcmv1Provider endpoint with WireMock"""
    test_id = "guardian.factors.push_notification.update_fcmv_1_provider.0"
    client = get_client(test_id)
    client.guardian.factors.push_notification.update_fcmv_1_provider()
    verify_request_count(test_id, "PATCH", "/guardian/factors/push-notification/providers/fcmv1", None, 1)


def test_guardian_factors_pushNotification_get_sns_provider() -> None:
    """Test getSnsProvider endpoint with WireMock"""
    test_id = "guardian.factors.push_notification.get_sns_provider.0"
    client = get_client(test_id)
    client.guardian.factors.push_notification.get_sns_provider()
    verify_request_count(test_id, "GET", "/guardian/factors/push-notification/providers/sns", None, 1)


def test_guardian_factors_pushNotification_set_sns_provider() -> None:
    """Test setSnsProvider endpoint with WireMock"""
    test_id = "guardian.factors.push_notification.set_sns_provider.0"
    client = get_client(test_id)
    client.guardian.factors.push_notification.set_sns_provider()
    verify_request_count(test_id, "PUT", "/guardian/factors/push-notification/providers/sns", None, 1)


def test_guardian_factors_pushNotification_update_sns_provider() -> None:
    """Test updateSnsProvider endpoint with WireMock"""
    test_id = "guardian.factors.push_notification.update_sns_provider.0"
    client = get_client(test_id)
    client.guardian.factors.push_notification.update_sns_provider()
    verify_request_count(test_id, "PATCH", "/guardian/factors/push-notification/providers/sns", None, 1)


def test_guardian_factors_pushNotification_get_selected_provider() -> None:
    """Test getSelectedProvider endpoint with WireMock"""
    test_id = "guardian.factors.push_notification.get_selected_provider.0"
    client = get_client(test_id)
    client.guardian.factors.push_notification.get_selected_provider()
    verify_request_count(test_id, "GET", "/guardian/factors/push-notification/selected-provider", None, 1)


def test_guardian_factors_pushNotification_set_provider() -> None:
    """Test setProvider endpoint with WireMock"""
    test_id = "guardian.factors.push_notification.set_provider.0"
    client = get_client(test_id)
    client.guardian.factors.push_notification.set_provider(
        provider="guardian",
    )
    verify_request_count(test_id, "PUT", "/guardian/factors/push-notification/selected-provider", None, 1)
